@@ -174,6 +174,12 @@ def format_permil(permil, machine_friendly):
 
     return format_ratio(float(permil) / 1000, machine_friendly)
 
+def format_bp(bp, machine_friendly):
+    if machine_friendly:
+        return '%f' % bp
+
+    return format_ratio(float(bp) / 10000, machine_friendly)
+
 def indent_lines(string, indent_width):
     return '\n'.join([' ' * indent_width + l for l in string.split('\n')])
 
@@ -313,6 +319,13 @@ def text_to_ratio(txt):
         ratio /= 100.0
     return ratio
 
+def text_to_bp(txt):
+    success, number = try_common_input(txt, 0)
+    if success:
+        return number
+
+    return text_to_ratio(txt) * 10000
+
 def text_to_permil(txt):
     success, number = try_common_input(txt, 0)
     if success:
