@@ -6,6 +6,7 @@ Contains core functions for DAMON control.
 
 import collections
 import copy
+import json
 import os
 import random
 import time
@@ -194,6 +195,7 @@ class DamonRegion:
     # nr_accesses and age could be None
     nr_accesses = None
     age = None
+    scheme = None # non-None if tried region
 
     def __init__(self, start, end, nr_accesses=None, nr_accesses_unit=None,
             age=None, age_unit=None):
@@ -263,6 +265,7 @@ class DamonRegion:
 class DamonTarget:
     pid = None
     regions = None
+    context = None
 
     def __init__(self, pid, regions):
         self.pid = pid
@@ -405,6 +408,51 @@ class DamosAccessPattern:
                 other.converted_for_units(
                     unit_samples, unit_aggr_intervals, intervals))
 
+class DamosQuotaGoal:
+    metric = None
+    target_value = None
+    current_value = None
+    quotas = None
+
+    def __init__(self, metric='user_input',
+                 target_value='0', current_value='0'):
+        self.metric = metric
+        # todo: support time string for some_mem_psi_us
+        self.target_value = _damo_fmt_str.text_to_nr(target_value)
+        self.current_value = _damo_fmt_str.text_to_nr(current_value)
+
+    def to_str(self, raw):
+        return 'metric %s target %s current %s' % (
+                self.metric,
+                _damo_fmt_str.format_nr(self.target_value, raw),
+                _damo_fmt_str.format_nr(self.current_value, raw),)
+
+    def __str__(self):
+        return self.to_str(False)
+
+    def __eq__(self, other):
+        return (type(self) == type(other) and self.metric == other.metric and
+                self.target_value == other.target_value and
+                self.current_value == other.current_value)
+
+    @classmethod
+    def from_kvpairs(cls, kv):
+        if 'target_value_bp' in kv:
+            # For supporing old version of bad naming.  Should deprecate later.
+            return DamosQuotaGoal(target_value=kv['target_value_bp'],
+                                  current_value=kv['current_value_bp'])
+        return DamosQuotaGoal(
+                metric=kv['metric'], target_value=kv['target_value'],
+                current_value=kv['current_value'])
+
+    def to_kvpairs(self, raw=False):
+        return collections.OrderedDict([
+            ('metric', self.metric),
+            ('target_value', _damo_fmt_str.format_nr(self.target_value,
+                raw)),
+            ('current_value', _damo_fmt_str.format_nr(self.current_value,
+                raw))])
+
 class DamosQuotas:
     time_ms = None
     sz_bytes = None
@@ -412,9 +460,12 @@ class DamosQuotas:
     weight_sz_permil = None
     weight_nr_accesses_permil = None
     weight_age_permil = None
+    goals = None
+    effective_sz_bytes = None
+    scheme = None
 
     def __init__(self, time_ms=0, sz_bytes=0, reset_interval_ms='max',
-            weights=['0 %', '0 %', '0 %']):
+            weights=['0 %', '0 %', '0 %'], goals=[], effective_sz_bytes=0):
         self.time_ms = _damo_fmt_str.text_to_ms(time_ms)
         self.sz_bytes = _damo_fmt_str.text_to_bytes(sz_bytes)
         self.reset_interval_ms = _damo_fmt_str.text_to_ms(reset_interval_ms)
@@ -422,6 +473,11 @@ class DamosQuotas:
         self.weight_nr_accesses_permil = _damo_fmt_str.text_to_permil(
                 weights[1])
         self.weight_age_permil = _damo_fmt_str.text_to_permil(weights[2])
+        self.goals = goals
+        self.effective_sz_bytes = _damo_fmt_str.text_to_bytes(
+                effective_sz_bytes)
+        for goal in self.goals:
+            goal.quotas = self
 
     def __str__(self):
         return self.to_str(False)
@@ -432,28 +488,38 @@ class DamosQuotas:
                 other.reset_interval_ms and self.weight_sz_permil ==
                 other.weight_sz_permil and self.weight_nr_accesses_permil ==
                 other.weight_nr_accesses_permil and self.weight_age_permil ==
-                other.weight_age_permil)
+                other.weight_age_permil and self.goals == other.goals)
 
     @classmethod
     def from_kvpairs(cls, kv):
+        if 'goals' in kv:
+            goals = [DamosQuotaGoal.from_kvpairs(goal) for goal in kv['goals']]
+        else:
+            goals = []
         return DamosQuotas(kv['time_ms'], kv['sz_bytes'],
                 kv['reset_interval_ms'],
                 [kv['weights']['sz_permil'],
                     kv['weights']['nr_accesses_permil'],
-                    kv['weights']['age_permil'],])
+                    kv['weights']['age_permil'],],
+                goals,
+                kv['effective_sz_bytes'] if 'effective_sz_bytes' in kv else 0)
 
     def to_str(self, raw):
-        return '\n'.join([
-            '%s / %s per %s' % (
+        lines = [
+            '%s / %s / %s per %s' % (
                 _damo_fmt_str.format_time_ns(self.time_ms * 1000000, raw),
-                _damo_fmt_str.format_time_ns(self.sz_bytes, raw),
-                _damo_fmt_str.format_time_ms(self.reset_interval_ms, raw)),
+                _damo_fmt_str.format_sz(self.sz_bytes, raw),
+                _damo_fmt_str.format_sz(self.effective_sz_bytes, raw),
+                _damo_fmt_str.format_time_ms(self.reset_interval_ms, raw))]
+        for idx, goal in enumerate(self.goals):
+            lines.append('goal %d: %s' % (idx, goal.to_str(raw)))
+        lines.append(
             'priority: sz %s, nr_accesses %s, age %s' % (
                 _damo_fmt_str.format_permil(self.weight_sz_permil, raw),
                 _damo_fmt_str.format_permil(
                     self.weight_nr_accesses_permil, raw),
-                _damo_fmt_str.format_permil(self.weight_age_permil, raw)),
-            ])
+                _damo_fmt_str.format_permil(self.weight_age_permil, raw)))
+        return '\n'.join(lines)
 
     def to_kvpairs(self, raw=False):
         return collections.OrderedDict([
@@ -461,6 +527,9 @@ class DamosQuotas:
             ('sz_bytes', _damo_fmt_str.format_sz(self.sz_bytes, raw)),
             ('reset_interval_ms', _damo_fmt_str.format_time_ms_exact(
                 self.reset_interval_ms, raw)),
+            ('goals', [goal.to_kvpairs(raw) for goal in self.goals]),
+            ('effective_sz_bytes',
+             _damo_fmt_str.format_sz(self.effective_sz_bytes, raw)),
             ('weights', (collections.OrderedDict([
                 ('sz_permil',
                     _damo_fmt_str.format_permil(self.weight_sz_permil, raw)),
@@ -538,6 +607,7 @@ class DamosFilter:
     memcg_path = None
     address_range = None    # DamonRegion
     damon_target_idx = None
+    scheme = None
 
     def __init__(self, filter_type, matching, memcg_path=None,
             address_range=None, damon_target_idx=None):
@@ -594,7 +664,8 @@ class DamosStats:
     sz_applied = None
     qt_exceeds = None
 
-    def __init__(self, nr_tried, sz_tried, nr_applied, sz_applied, qt_exceeds):
+    def __init__(self, nr_tried=0, sz_tried=0, nr_applied=0, sz_applied=0,
+            qt_exceeds=0):
         self.nr_tried = nr_tried
         self.sz_tried = sz_tried
         self.nr_applied = nr_applied
@@ -613,6 +684,15 @@ class DamosStats:
     def __str__(self):
         return self.to_str(False)
 
+    def to_kvpairs(self, raw=False):
+        kv = collections.OrderedDict()
+        kv['nr_tried'] = _damo_fmt_str.format_nr(self.nr_tried, raw)
+        kv['sz_tried'] = _damo_fmt_str.format_sz(self.sz_tried, raw)
+        kv['nr_applied'] = _damo_fmt_str.format_nr(self.nr_applied, raw)
+        kv['sz_applied'] = _damo_fmt_str.format_sz(self.sz_applied, raw)
+        kv['qt_exceeds'] = _damo_fmt_str.format_nr(self.qt_exceeds, raw)
+        return kv
+
 # TODO: check support of pageout and lru_(de)prio
 damos_actions = [
         'willneed',
@@ -637,15 +717,18 @@ damos_action_stat = damos_actions[7]
 class Damos:
     access_pattern = None
     action = None
+    apply_interval_us = None
     quotas = None
     watermarks = None
     filters = None
     stats = None
     tried_regions = None
     tried_bytes = None
+    context = None
 
     # for monitoring only by default
     def __init__(self, access_pattern=None, action=damos_action_stat,
+            apply_interval_us=None,
             quotas=None, watermarks=None, filters=None, stats=None,
             tried_regions=None, tried_bytes=None):
         self.access_pattern = (access_pattern
@@ -653,14 +736,22 @@ class Damos:
         if not action in damos_actions:
             raise Exception('wrong damos action: %s' % action)
         self.action = action
+        if apply_interval_us != None:
+            self.apply_interval_us = _damo_fmt_str.text_to_us(
+                    apply_interval_us)
+        else:
+            self.apply_interval_us = 0
         self.quotas = quotas if quotas != None else DamosQuotas()
+        self.quotas.scheme = self
         self.watermarks = (watermarks
                 if watermarks != None else DamosWatermarks())
         self.filters = filters if filters != None else []
-        self.stats = stats
-        self.tried_regions = tried_regions
-        if self.tried_regions == None:
-            self.tried_regions = []
+        for filter_ in self.filters:
+            filter_.scheme = self
+        self.stats = stats if stats != None else DamosStats()
+        self.tried_regions = tried_regions if tried_regions != None else []
+        for tried_region in self.tried_regions:
+            tried_region.scheme = self
         self.tried_bytes = 0
         if tried_bytes:
             self.tried_bytes = _damo_fmt_str.text_to_bytes(
@@ -670,7 +761,9 @@ class Damos:
                 self.tried_bytes += region.size()
 
     def to_str(self, raw):
-        lines = ['action: %s' % self.action]
+        lines = ['action: %s per %s' % (self.action,
+            _damo_fmt_str.format_time_us(self.apply_interval_us, raw)
+            if self.apply_interval_us != 0 else 'aggr interval')]
         lines.append('target access pattern')
         lines.append(_damo_fmt_str.indent_lines(
             self.access_pattern.to_str(raw), 4))
@@ -702,7 +795,9 @@ class Damos:
     def __eq__(self, other):
         return (type(self) == type(other) and
                 self.access_pattern == other.access_pattern and
-                self.action == other.action and self.quotas == other.quotas and
+                self.action == other.action and
+                self.apply_interval_us == other.apply_interval_us and
+                self.quotas == other.quotas and
                 self.watermarks == other.watermarks and
                 self.filters == other.filters)
 
@@ -715,6 +810,7 @@ class Damos:
         return Damos(DamosAccessPattern.from_kvpairs(kv['access_pattern'])
                     if 'access_pattern' in kv else DamosAccessPattern(),
                 kv['action'] if 'action' in kv else damos_action_stat,
+                kv['apply_interval_us'] if 'apply_interval_us' in kv else None,
                 DamosQuotas.from_kvpairs(kv['quotas'])
                     if 'quotas' in kv else DamosQuotas(),
                 DamosWatermarks.from_kvpairs(kv['watermarks'])
@@ -726,43 +822,53 @@ class Damos:
         kv = collections.OrderedDict()
         kv['action'] = self.action
         kv['access_pattern'] = self.access_pattern.to_kvpairs(raw)
+        kv['apply_interval_us'] = self.apply_interval_us
         kv['quotas'] = self.quotas.to_kvpairs(raw)
         kv['watermarks'] = self.watermarks.to_kvpairs(raw)
         filters = []
         for damos_filter in self.filters:
             filters.append(damos_filter.to_kvpairs(raw))
         kv['filters'] = filters
+        if self.stats != None:
+            kv['stats'] = self.stats.to_kvpairs(raw)
         return kv
 
     def effectively_equal(self, other, intervals):
         return (type(self) == type(other) and
                 self.access_pattern.effectively_equal(
                     other.access_pattern, intervals) and
-                self.action == other.action and self.quotas == other.quotas and
+                self.action == other.action and
+                self.apply_interval_us == other.apply_interval_us and
+                self.quotas == other.quotas and
                 self.watermarks == other.watermarks and
                 self.filters == other.filters)
 
 class DamonCtx:
-    intervals = None
-    nr_regions = None
     ops = None
     targets = None
+    intervals = None
+    nr_regions = None
     schemes = None
+    kdamond = None
 
-    def __init__(self, intervals, nr_regions, ops, targets, schemes):
-        self.intervals = intervals
-        self.nr_regions = nr_regions
+    def __init__(self, ops, targets, intervals, nr_regions, schemes):
         self.ops = ops
         self.targets = targets
+        for target in self.targets:
+            target.context = self
+        self.intervals = intervals
+        self.nr_regions = nr_regions
         self.schemes = schemes
+        for scheme in self.schemes:
+            scheme.context = self
 
     def to_str(self, raw):
         lines = ['ops: %s' % self.ops]
-        lines.append('intervals: %s' % self.intervals.to_str(raw))
-        lines.append('nr_regions: %s' % self.nr_regions.to_str(raw))
         for idx, target in enumerate(self.targets):
             lines.append('target %d' % idx)
             lines.append(_damo_fmt_str.indent_lines(target.to_str(raw), 4))
+        lines.append('intervals: %s' % self.intervals.to_str(raw))
+        lines.append('nr_regions: %s' % self.nr_regions.to_str(raw))
         for idx, scheme in enumerate(self.schemes):
             lines.append('scheme %d' % idx)
             lines.append(_damo_fmt_str.indent_lines(scheme.to_str(raw), 4))
@@ -780,22 +886,22 @@ class DamonCtx:
     @classmethod
     def from_kvpairs(cls, kv):
         ctx = DamonCtx(
+                kv['ops'],
+                [DamonTarget.from_kvpairs(t) for t in kv['targets']],
                 DamonIntervals.from_kvpairs(kv['intervals'])
                     if 'intervals' in kv else DamonIntervals(),
                 DamonNrRegionsRange.from_kvpairs(kv['nr_regions'])
                     if 'nr_regions' in kv else DAmonNrRegionsRange(),
-                kv['ops'],
-                [DamonTarget.from_kvpairs(t) for t in kv['targets']],
                 [Damos.from_kvpairs(s) for s in kv['schemes']]
                     if 'schemes' in kv else [])
         return ctx
 
     def to_kvpairs(self, raw=False):
         kv = collections.OrderedDict({})
-        kv['intervals'] = self.intervals.to_kvpairs(raw)
-        kv['nr_regions'] = self.nr_regions.to_kvpairs(raw)
         kv['ops'] = self.ops
         kv['targets'] = [t.to_kvpairs(raw) for t in self.targets]
+        kv['intervals'] = self.intervals.to_kvpairs(raw)
+        kv['nr_regions'] = self.nr_regions.to_kvpairs(raw)
         kv['schemes'] = [s.to_kvpairs(raw) for s in self.schemes]
         return kv
 
@@ -811,6 +917,8 @@ class Kdamond:
         self.state = state
         self.pid = pid
         self.contexts = contexts
+        for ctx in self.contexts:
+            ctx.kdamond = self
 
     def summary_str(self):
         return 'state: %s, pid: %s' % (self.state, self.pid)
@@ -848,9 +956,21 @@ class Kdamond:
 import _damo_fs
 import _damon_dbgfs
 import _damon_sysfs
+import damo_version
 
 # System check
 
+# damo supports all DAMON-enabled kernels.  For that, damo maintains list of
+# the DAMON features, and a dict saying whether the feature is supported on the
+# running kernel.  Since the supports depend on underlying DAMON interface, the
+# dict is populated by _damon_fs, and saved as _damon_fs.feature_supports.
+#
+# The feature_supports population cannot be fully done while DAMON is running,
+# particularly in case of debugfs.  Sysfs is ok for now, but similar issue
+# could happen in future.  For the reason, the feature_supports are saved at
+# feature_supports_file_path file.  damo commands can ask
+# _damon.ensure_initialized() to load/save the file.
+
 features = ['record',       # was in DAMON patchset, but not merged in mainline
             'vaddr',        # merged in v5.15, thebeginning
             'schemes',      # merged in v5.16
@@ -866,7 +986,16 @@ features = ['record',       # was in DAMON patchset, but not merged in mainline
             'fvaddr',       # merged in v5.19 (b82434471cd2)
             'schemes_tried_regions',    # merged in v6.2-rc1
             'schemes_filters',          # merged in v6.3-rc1
-            'schemes_tried_regions_sz', # developing
+            'schemes_filters_anon',     # merged in v6.3-rc1
+            'schemes_filters_memcg',    # merged in v6.3-rc1
+            'schemes_tried_regions_sz', # merged in v6.6-rc1
+            'schemes_filters_addr',     # merged in v6.6-rc1
+            'schemes_filters_target',   # merged in v6.6-rc1
+            'schemes_apply_interval',   # merged in v6.7-rc1
+            'schemes_quota_goals',      # merged in v6.8-rc1
+            'schemes_quota_effective_bytes',    # in development
+            'schemes_quota_goal_metric',    # in development
+            'schemes_quota_goal_some_psi',  # in development
             ]
 
 _damon_fs = None
@@ -878,42 +1007,116 @@ def ensure_root_permission():
         print('Run as root')
         exit(1)
 
+feature_supports_file_path = os.path.join(os.environ['HOME'],
+        '.damo.damon_feature_supports')
+
+feature_support_file_format_ver = 1
+
+def read_feature_supports_file():
+    '''Return error string'''
+    if not os.path.isfile(feature_supports_file_path):
+        return '%s not exist' % feature_supports_file_path
+    try:
+        with open(feature_supports_file_path, 'r') as f:
+            feature_supports = json.load(f)
+    except Exception as e:
+        return 'reading feature supports failed (%s)' % e
+    if not 'file_format_ver' in feature_supports:
+        # The initial format
+        set_feature_supports(feature_supports)
+        return None
+    file_format_ver = feature_supports['file_format_ver']
+    # support only the init version and current version for now.
+    if file_format_ver != feature_support_file_format_ver:
+        return 'unsupported format version %s' % file_format_ver
+    if not damon_interface() in feature_supports:
+        return 'no feature_supports for %s interface saved' % damon_interface()
+    set_feature_supports(feature_supports[damon_interface()])
+
+def write_feature_supports_file():
+    '''Return error string'''
+    feature_supports, err = get_feature_supports()
+    if err != None:
+        return 'get_feature_supports() failed (%s)' % err
+
+    to_save = {}
+    if os.path.isfile(feature_supports_file_path):
+        with open(feature_supports_file_path, 'r') as f:
+            try:
+                to_save = json.load(f)
+            except:
+                # Maybe previous writing was something wrong.  Just overwirte.
+                to_save = {}
+        if not 'file_format_ver' in to_save:
+            to_save = {}
+        elif to_save['file_format_ver'] != feature_support_file_format_ver:
+            to_save = {}
+
+    if to_save == {}:
+        to_save['file_format_ver'] = feature_support_file_format_ver
+    to_save[damon_interface()] = feature_supports
+
+    with open(feature_supports_file_path, 'w') as f:
+        json.dump(to_save, f, indent=4, sort_keys=True)
+
 def feature_supported(feature):
     return _damon_fs.feature_supported(feature)
 
-def initialize(args):
+def get_feature_supports():
+    err = _damon_fs.update_supported_features()
+    if err != None:
+        return None, err
+    return _damon_fs.feature_supports, None
+
+def set_feature_supports(feature_supports):
+    _damon_fs.feature_supports = feature_supports
+
+def initialize(damon_interface, debug_damon,
+        save_feature_supports, load_feature_supports):
     global _damon_fs
-    if args.damon_interface == 'sysfs':
+    if damon_interface == 'sysfs':
         _damon_fs = _damon_sysfs
-    elif args.damon_interface == 'debugfs':
+    elif damon_interface == 'debugfs':
         _damon_fs = _damon_dbgfs
-    elif args.damon_interface == 'auto':
+    elif damon_interface == 'auto':
         if _damon_sysfs.supported():
             _damon_fs = _damon_sysfs
         else:
             _damon_fs = _damon_dbgfs
+    if not _damon_fs.supported():
+        return 'DAMON interface (%s) not supported' % damon_interface
 
     global pr_debug_log
-    if args.debug_damon:
+    if debug_damon:
         pr_debug_log = True
 
-    return _damon_fs.update_supported_features()
+    if load_feature_supports:
+        err = read_feature_supports_file()
+        if err:
+            return err
+
+    if save_feature_supports:
+        write_feature_supports_file()
+
+    return None
 
 initialized = False
-def ensure_initialized(args):
+def ensure_initialized(args, save_feature_supports, load_feature_supports):
     global initialized
 
     if initialized:
         return
-    err = initialize(args)
+    err = initialize(args.damon_interface_DEPRECATED, args.debug_damon,
+            save_feature_supports, load_feature_supports)
     if err != None:
         print(err)
         exit(1)
     initialized = True
 
-def ensure_root_and_initialized(args):
+def ensure_root_and_initialized(args, save_feature_supports=False,
+        load_feature_supports=False):
     ensure_root_permission()
-    ensure_initialized(args)
+    ensure_initialized(args, save_feature_supports, load_feature_supports)
 
 def damon_interface():
     if _damon_fs == _damon_sysfs:
@@ -932,11 +1135,26 @@ def commit_staged(kdamond_idxs):
         return 'debugfs interface does not support commit_staged()'
     return _damon_fs.commit_staged(kdamond_idxs)
 
-def commit(kdamonds):
-    err = stage_kdamonds(kdamonds)
-    if err:
-        return 'staging updates failed (%s)' % err
-    err = commit_staged(['%s' % idx for idx, k in enumerate(kdamonds)])
+def commit_quota_goals(kdamond_idxs):
+    if _damon_fs == _damon_dbgfs:
+        return 'debugfs interface does not support commit_quota_goals()'
+    return _damon_fs.commit_quota_goals(kdamond_idxs)
+
+def commit(kdamonds, commit_quota_goals_only=False):
+    if not commit_quota_goals_only:
+        err = stage_kdamonds(kdamonds)
+        if err:
+            return 'staging updates failed (%s)' % err
+
+    kdamond_idxs = ['%s' % idx for idx, k in enumerate(kdamonds)]
+
+    if commit_quota_goals_only:
+        err = commit_quota_goals(kdamond_idxs)
+        if err:
+            return 'commit quotas failed (%s)' % err
+        return None
+
+    err = commit_staged(kdamond_idxs)
     if err:
         return 'commit staged updates filed (%s)' % err
     return None
@@ -956,7 +1174,13 @@ def update_schemes_tried_regions(kdamond_idxs=None):
         kdamond_idxs = running_kdamond_idxs()
     return _damon_fs.update_schemes_tried_regions(kdamond_idxs)
 
-def update_schemes_status(stats=True, tried_regions=True):
+def update_schemes_quota_effective_bytes(kdamond_idxs=None):
+    if kdamond_idxs == None:
+        kdamond_idxs = running_kdamond_idxs()
+    return _damon_fs.update_schemes_quota_effective_bytes(kdamond_idxs)
+
+def update_schemes_status(stats=True, tried_regions=True,
+                          quota_effective_bytes=False):
     '''Returns error string or None'''
     idxs = running_kdamond_idxs()
     if len(idxs) == 0:
@@ -966,7 +1190,13 @@ def update_schemes_status(stats=True, tried_regions=True):
         if err != None:
             return err
     if tried_regions and feature_supported('schemes_tried_regions'):
-        return update_schemes_tried_regions(idxs)
+        err = update_schemes_tried_regions(idxs)
+        if err != None:
+            return err
+
+    if quota_effective_bytes and feature_supported('schemes_quota_effective_bytes'):
+        return update_schemes_quota_effective_bytes(idxs)
+
     return None
 
 def turn_damon_on(kdamonds_idxs):
@@ -989,11 +1219,13 @@ def is_kdamond_running(kdamond_idx):
 def current_kdamonds():
     return _damon_fs.current_kdamonds()
 
-def update_read_kdamonds(nr_retries=0):
+def update_read_kdamonds(nr_retries=0, update_stats=True,
+        update_tried_regions=True, update_quota_effective_bytes=False):
     err = 'assumed error'
     nr_tries = 0
     while True:
-        err = update_schemes_status()
+        err = update_schemes_status(update_stats, update_tried_regions,
+                                    update_quota_effective_bytes)
         nr_tries += 1
         if err == None or nr_tries > nr_retries:
             break
