@@ -33,7 +33,10 @@ def init_regions_for(args):
 
     if args.ops == 'paddr' and not init_regions:
         if args.numa_node != None:
-            init_regions = _damo_paddr_layout.paddr_region_of(args.numa_node)
+            init_regions, err = _damo_paddr_layout.paddr_region_of(
+                    args.numa_node)
+            if err != None:
+                return None, err
         else:
             init_regions = [_damo_paddr_layout.default_paddr_region()]
         try:
@@ -122,13 +125,52 @@ def damos_options_to_filters(filters_args):
             return None, 'unsupported filter type'
     return filters, None
 
-def damos_options_to_scheme(sz_region, access_rate, age, action, quotas,
-        wmarks, filters):
+def damos_quotas_cons_arg(cmd_args):
+    time_ms = 0
+    sz_bytes = 0
+    reset_interval_ms = 'max'
+    weights = ['0 %', '0 %', '0 %']
+
+    nr_cmd_args = len(cmd_args)
+    if nr_cmd_args >= 1:
+        time_ms = cmd_args[0]
+    if nr_cmd_args >= 2:
+        sz_bytes = cmd_args[1]
+    if nr_cmd_args >= 3:
+        reset_interval_ms = cmd_args[2]
+    if nr_cmd_args >= 4:
+        weights[0] = cmd_args[3]
+    if nr_cmd_args >= 5:
+        weights[1] = cmd_args[4]
+    if nr_cmd_args >= 6:
+        weights[2] = cmd_args[5]
+
+    return [time_ms, sz_bytes, reset_interval_ms, weights]
+
+def damos_options_to_scheme(sz_region, access_rate, age, action,
+        apply_interval, quotas, goals, wmarks, filters):
     if quotas != None:
-        qargs = quotas
+        gargs = goals
+        # garg should be <metric> <target value> [current value]
+        # [current value] is given for only 'user_input' <metric>
+        for garg in gargs:
+            if not len(garg) in [2, 3]:
+                return None, 'Wrong --damos_quota_goal (%s)' % garg
+            if garg[0] == 'user_input' and len(garg) != 3:
+                return None, 'Wrong --damos_quota_goal (%s)' % garg
+            if garg[0] != 'user_input' and len(garg) != 2:
+                return None, 'Wrong --damos_quota_goal (%s)' % garg
         try:
-            quotas = _damon.DamosQuotas(qargs[0], qargs[1], qargs[2],
-                    [qargs[3], qargs[4], qargs[5]])
+            goals = [_damon.DamosQuotaGoal(*garg) for garg in gargs]
+        except Exception as e:
+            return None, 'Wrong --damos_quota_goal (%s, %s)' % (gargs, e)
+
+        qargs = quotas
+        if len(qargs) > 6:
+            return None, 'Wrong --damos_quotas (%s, >6 parameters)' % qargs
+        try:
+            quotas = _damon.DamosQuotas(*damos_quotas_cons_arg(qargs),
+                                        goals=goals)
         except Exception as e:
             return None, 'Wrong --damos_quotas (%s, %s)' % (qargs, e)
 
@@ -148,8 +190,8 @@ def damos_options_to_scheme(sz_region, access_rate, age, action, quotas,
         return _damon.Damos(
                 access_pattern=_damon.DamosAccessPattern(sz_region,
                     access_rate, _damon.unit_percent, age, _damon.unit_usec),
-                action=action, quotas=quotas, watermarks=wmarks,
-                filters=filters), None
+                action=action, apply_interval_us=apply_interval, quotas=quotas,
+                watermarks=wmarks, filters=filters), None
     except Exception as e:
         return None, 'Wrong \'--damos_*\' argument (%s)' % e
 
@@ -161,8 +203,17 @@ def damos_options_to_schemes(args):
         return [], 'too much --damos_access_rate'
     if len(args.damos_age) > nr_schemes:
         return [], 'too much --damos_age'
+    if len(args.damos_apply_interval) > nr_schemes:
+        return [], 'too much --damos_apply_interval'
     if len(args.damos_quotas) > nr_schemes:
         return [], 'too much --damos_quotas'
+    if len(args.damos_quota_goal) > 0 and nr_schemes > 1:
+        if len(args.damos_nr_quota_goals) == 0:
+            return [], '--damos_nr_quota_goals required'
+    if nr_schemes == 1 and args.damos_nr_quota_goals == []:
+        args.damos_nr_quota_goals = [len(args.damos_quota_goal)]
+    if sum(args.damos_nr_quota_goals) != len(args.damos_quota_goal):
+        return [], 'wrong --damos_nr_quota_goals'
     if len(args.damos_wmarks) > nr_schemes:
         return [], 'too much --damos_wmarks'
     # for multiple schemes, number of filters per scheme is required
@@ -179,10 +230,18 @@ def damos_options_to_schemes(args):
     args.damos_access_rate += [['min', 'max']] * (
             nr_schemes - len(args.damos_access_rate))
     args.damos_age += [['min', 'max']] * (nr_schemes - len(args.damos_age))
+    args.damos_apply_interval += [0] * (
+            nr_schemes - len(args.damos_apply_interval))
     args.damos_quotas += [None] * (nr_schemes - len(args.damos_quotas))
     args.damos_wmarks += [None] * (nr_schemes - len(args.damos_wmarks))
     schemes = []
     for i in range(nr_schemes):
+        qgoals = []
+        if args.damos_quota_goal:
+            goal_start_index = sum(args.damos_nr_quota_goals[:i])
+            goal_end_index = goal_start_index + args.damos_nr_quota_goals[i]
+            qgoals = args.damos_quota_goal[goal_start_index:goal_end_index]
+
         filters = []
         if args.damos_filter:
             filter_start_index = sum(args.damos_nr_filters[:i])
@@ -190,8 +249,8 @@ def damos_options_to_schemes(args):
             filters = args.damos_filter[filter_start_index:filter_end_index]
         scheme, err = damos_options_to_scheme(args.damos_sz_region[i],
                 args.damos_access_rate[i], args.damos_age[i],
-                args.damos_action[i], args.damos_quotas[i],
-                args.damos_wmarks[i], filters)
+                args.damos_action[i], args.damos_apply_interval[i],
+                args.damos_quotas[i], qgoals, args.damos_wmarks[i], filters)
         if err != None:
             return [], err
         schemes.append(scheme)
@@ -238,7 +297,7 @@ def damon_ctx_for(args):
         return None, err
 
     try:
-        ctx = _damon.DamonCtx(intervals, nr_regions, ops, [target], schemes)
+        ctx = _damon.DamonCtx(ops, [target], intervals, nr_regions, schemes)
         return ctx, None
     except Exception as e:
         return None, 'Creating context from arguments failed (%s)' % e
@@ -251,7 +310,8 @@ def kdamonds_from_json_arg(arg):
         else:
             kdamonds_str = arg
         kdamonds_kvpairs = json.loads(kdamonds_str)['kdamonds']
-        return [kdamond.from_kvpairs(kvp) for kvp in kdamonds_kvpairs], None
+        return [_damon.Kdamond.from_kvpairs(kvp)
+                for kvp in kdamonds_kvpairs], None
     except Exception as e:
         return None, e
 
@@ -349,11 +409,11 @@ def stage_kdamonds(args):
         return None, 'cannot apply kdamonds from args (%s)' % err
     return kdamonds, None
 
-def commit_kdamonds(args):
+def commit_kdamonds(args, commit_quota_goals_only):
     kdamonds, err = kdamonds_for(args)
     if err:
         return None, 'cannot create kdamonds to commit from args (%s)' % err
-    err = _damon.commit(kdamonds)
+    err = _damon.commit(kdamonds, commit_quota_goals_only)
     if err:
         return None, 'cannot commit kdamonds (%s)' % err
     return kdamonds, None
@@ -368,14 +428,14 @@ def turn_damon_on(args):
 # Commandline options setup helpers
 
 def set_common_argparser(parser):
-    parser.add_argument('--damon_interface',
+    parser.add_argument('--damon_interface_DEPRECATED',
             choices=['sysfs', 'debugfs', 'auto'],
             default='auto',
             help='underlying DAMON interface to use (!! DEPRECATED)')
     parser.add_argument('--debug_damon', action='store_true',
             help='Print debugging log')
 
-def set_monitoring_attrs_argparser(parser):
+def set_monitoring_attrs_pinpoint_argparser(parser):
     # for easier pinpoint setup
     parser.add_argument('-s', '--sample', metavar='<microseconds>',
             default=5000, help='sampling interval (us)')
@@ -388,6 +448,7 @@ def set_monitoring_attrs_argparser(parser):
     parser.add_argument('-m', '--maxr', metavar='<# regions>',
             default=1000, help='maximum number of regions')
 
+def set_monitoring_attrs_argparser(parser):
     # for easier total setup
     parser.add_argument('--monitoring_intervals', nargs=3,
             default=['5ms', '100ms', '1s'],
@@ -398,13 +459,20 @@ def set_monitoring_attrs_argparser(parser):
             help='min/max number of monitoring regions')
 
 def set_monitoring_argparser(parser):
-    set_monitoring_attrs_argparser(parser)
+    parser.add_argument('--ops', choices=['vaddr', 'paddr', 'fvaddr'],
+            help='monitoring operations set')
+    parser.add_argument('--target_pid', type=int, metavar='<pid>',
+            help='pid of monitoring target process')
     parser.add_argument('-r', '--regions', metavar='"<start>-<end> ..."',
             type=str, default='', help='monitoring target address regions')
     parser.add_argument('--numa_node', metavar='<node id>', type=int,
             help='if target is \'paddr\', limit it to the numa node')
+    set_monitoring_attrs_argparser(parser)
 
 def set_damos_argparser(parser):
+    parser.add_argument('--damos_action', metavar='<action>',
+            choices=_damon.damos_actions, action='append', default=[],
+            help='damos action to apply to the target regions')
     parser.add_argument('--damos_sz_region', metavar=('<min>', '<max>'),
             nargs=2, default=[], action='append',
             help='min/max size of damos target regions (bytes)')
@@ -414,15 +482,23 @@ def set_damos_argparser(parser):
     parser.add_argument('--damos_age', metavar=('<min>', '<max>'), nargs=2,
             default=[], action='append',
             help='min/max age of damos target regions (microseconds)')
-    parser.add_argument('--damos_action', metavar='<action>',
-            choices=_damon.damos_actions, action='append', default=[],
-            help='damos action to apply to the target regions')
+    parser.add_argument('--damos_apply_interval', metavar='<microseconds>',
+            action='append', default=[],
+            help='the apply interval for the scheme')
     parser.add_argument('--damos_quotas', default=[],
-            metavar=('<time (ms)>', '<size (bytes)>', '<reset interval (ms)>',
-                '<size priority weight (permil)>',
-                '<access rate priority weight> (permil)',
-                '<age priority weight> (permil)'), nargs=6, action='append',
-            help='damos quotas')
+            metavar='<quota parameter>', nargs='+', action='append',
+            help=' '.join([
+            'damos quotas (<time (ms)> [<size (bytes)> [<reset interval (ms)>',
+                '[<size priority weight (permil)>',
+                '[<access rate priority weight> (permil)',
+                '[<age priority weight> (permil)]]]]])']))
+    parser.add_argument('--damos_quota_goal', nargs='+', action='append',
+            default=[],
+            metavar='<metric or target value or current value>',
+            help='damos quota goal (<metric> <target value> [current value])')
+    parser.add_argument('--damos_nr_quota_goals', type=int, nargs='+',
+            default=[], metavar='<integer>',
+            help='number of quota goals for each scheme (in order)')
     parser.add_argument('--damos_wmarks', nargs=5, action='append', default=[],
             metavar=('<metric (none|free_mem_rate)>', '<interval (us)>',
                 '<high mark (permil)>', '<mid mark (permil)>',
@@ -440,19 +516,17 @@ def set_argparser(parser, add_record_options):
     if parser == None:
         parser = argparse.ArgumentParser()
     set_monitoring_argparser(parser)
-    parser.add_argument('--ops', choices=['vaddr', 'paddr', 'fvaddr'],
-            help='monitoring operations set')
-    parser.add_argument('--target_pid', type=int, help='target pid')
     set_damos_argparser(parser)
     parser.add_argument('-c', '--schemes', metavar='<json string or file>',
 	    help='data access monitoring-based operation schemes')
     parser.add_argument('--kdamonds', metavar='<json string or file>',
             help='json format kdamonds specification to run DAMON for')
     parser.add_argument('deducible_target', type=str,
-            metavar='<deducible target>', nargs='?',
-            help='the target (command, pid, or special keywords) to monitor')
+            metavar='<command, pid, special keywords, or kdamonds json spec>', nargs='?',
+            help='the implicit monitoring requests')
     if add_record_options:
         parser.add_argument('-o', '--out', metavar='<file path>', type=str,
                 default='damon.data', help='output file path')
     set_common_argparser(parser)
+    set_monitoring_attrs_pinpoint_argparser(parser)
     return parser
