@@ -1,14 +1,22 @@
 # SPDX-License-Identifier: GPL-2.0
 
 """
-Transform binary trace data into human readable text that can be used for
-heatmap drawing, or directly plot the data in a heatmap format.
+Transform DAMON monitoring results record into a heatmap.  The heatmap is
+constructed with pixels that each shows when (time), which memory region
+(space) was how frequently accessed (heat).  The time and space are represented
+by the location of the pixel on the map, while the heat is represented by it's
+color.
 
-Format of the text is:
+By default, the output shows the relative time, space, and heat values of each
+pixel of the map on each line, like below.
 
     <time> <space> <heat>
     ...
 
+By constructing the pixels based on the values, the user can draw more
+human-readable heatmap.  gnuplot like plot tools can be used.  If '--heatmap'
+option is givne, this tool does that on behalf of the human when '--heatmap'
+option is given.
 """
 
 import argparse
@@ -17,9 +25,11 @@ import subprocess
 import sys
 import tempfile
 
+import damo_record_info
+
 import _damo_ascii_color
 import _damo_fmt_str
-import _damon_result
+import _damon_records
 
 class HeatPixel:
     time = None
@@ -34,6 +44,17 @@ class HeatPixel:
 def add_heats(snapshot, duration, pixels, time_unit, space_unit, addr_range):
     """Add heats in a monitoring 'snapshot' of specific time 'duration' to
     the corresponding heats 'pixels'.
+
+    Args:
+      snapshot:     Data access monitoring results for a specific time
+                    (DamonSnapshot class).
+      duration:     The time of the snapshot to count and add heats to the
+                    pixels.
+      pixels:       The heatmap pixels that the heats of the snapshot of the
+                    duration will be added.
+      time_unit:    Time length that represented by each pixel
+      space_unit:   Memory address range size that represented by each pixel
+      addr_range:   The entire address range of the heatmap
     """
     pixel_sz = time_unit * space_unit
 
@@ -43,6 +64,10 @@ def add_heats(snapshot, duration, pixels, time_unit, space_unit, addr_range):
         if start >= end:
             continue
 
+        # The region and the corresponding pixel may not fit on the address
+        # space.  Get a fraction of the region that overlaps with the pixel,
+        # total heat (average heat * size of the fraction) of the fraction, and
+        # add it to the corresponding pixel in size-average heat.
         fraction_start = start
         addr_idx = int(float(fraction_start - addr_range[0]) / space_unit)
         while fraction_start < end:
@@ -74,6 +99,9 @@ def heat_pixels_from_snapshots(snapshots, time_range, addr_range, resols):
         start = shot.start_time
         end = min(shot.end_time, time_range[1])
 
+        # The snapshot's recorded time and the corresponding pixels row may not
+        # fit on the time space.  Get a fraction of the time that both
+        # overlaps, and add heats of the fractions to corresponding pixels.
         fraction_start = start
         time_idx = int(float(fraction_start - time_range[0]) / time_unit)
         while fraction_start < end:
@@ -84,7 +112,8 @@ def heat_pixels_from_snapshots(snapshots, time_range, addr_range, resols):
             time_idx += 1
     return pixels
 
-def heatmap_plot_ascii(pixels, time_range, addr_range, resols, colorset):
+def heatmap_plot_ascii(pixels, time_range, addr_range, resols, colorset,
+        print_colorset):
     highest_heat = None
     lowest_heat = None
     for snapshot in pixels:
@@ -106,7 +135,9 @@ def heatmap_plot_ascii(pixels, time_range, addr_range, resols, colorset):
                     (_damo_ascii_color.color_mode_start_txt(colorset, heat),
                         heat))
         print(''.join(chars) + _damo_ascii_color.color_mode_end_txt())
-    print('# access_frequency: %s' % _damo_ascii_color.color_samples(colorset))
+    if print_colorset:
+        print('# access_frequency: %s' %
+                _damo_ascii_color.color_samples(colorset))
     print('# x-axis: space (%d-%d: %s)' % (addr_range[0], addr_range[1],
         _damo_fmt_str.format_sz(addr_range[1] - addr_range[0], False)))
     print('# y-axis: time (%d-%d: %s)' % (time_range[0], time_range[1],
@@ -147,7 +178,8 @@ def pr_heats(args, __records):
 
         if args.heatmap == 'stdout':
             heatmap_plot_ascii(pixels, [tmin, tmax], [amin, amax],
-                    [tres, ares], args.stdout_heatmap_color)
+                    [tres, ares], args.stdout_heatmap_color, not
+                    args.stdout_heatmap_skip_color_example)
             return
 
         for row in pixels:
@@ -161,119 +193,10 @@ def pr_heats(args, __records):
 
                 print('%s\t%s\t%s' % (time, addr, pixel.heat))
 
-class GuideInfo:
-    tid = None
-    start_time = None
-    end_time = None
-    lowest_addr = None
-    highest_addr = None
-    gaps = None
-
-    def __init__(self, tid, start_time):
-        self.tid = tid
-        self.start_time = start_time
-        self.gaps = []
-
-    def regions(self):
-        regions = []
-        region = [self.lowest_addr]
-        for gap in self.gaps:
-            for idx, point in enumerate(gap):
-                if idx == 0:
-                    region.append(point)
-                    regions.append(region)
-                else:
-                    region = [point]
-        region.append(self.highest_addr)
-        regions.append(region)
-        return regions
-
-    def total_space(self):
-        ret = 0
-        for r in self.regions():
-            ret += r[1] - r[0]
-        return ret
-
-    def __str__(self):
-        lines = ['target_id:%d' % self.tid]
-        lines.append('time: %d-%d (%s)' % (self.start_time, self.end_time,
-                    _damo_fmt_str.format_time_ns(self.end_time - self.start_time,
-                        False)))
-        for idx, region in enumerate(self.regions()):
-            lines.append('region\t%2d: %020d-%020d (%s)' %
-                    (idx, region[0], region[1],
-                        _damo_fmt_str.format_sz(region[1] - region[0], False)))
-        return '\n'.join(lines)
-
-def is_overlap(region1, region2):
-    if region1[1] < region2[0]:
-        return False
-    if region2[1] < region1[0]:
-        return False
-    return True
-
-def overlap_region_of(region1, region2):
-    return [max(region1[0], region2[0]), min(region1[1], region2[1])]
-
-def overlapping_regions(regions1, regions2):
-    overlap_regions = []
-    for r1 in regions1:
-        for r2 in regions2:
-            if is_overlap(r1, r2):
-                r1 = overlap_region_of(r1, r2)
-        if r1:
-            overlap_regions.append(r1)
-    return overlap_regions
-
-def get_guide_info(records):
-    "return the set of guide information for the moitoring result"
-    guides = {}
-    for record in records:
-        for snapshot in record.snapshots:
-            monitor_time = snapshot.end_time
-            tid = record.target_id
-            if not tid in guides:
-                guides[tid] = GuideInfo(tid, monitor_time)
-            guide = guides[tid]
-            guide.end_time = monitor_time
-
-            last_addr = None
-            gaps = []
-            for r in snapshot.regions:
-                saddr = r.start
-                eaddr = r.end
-
-                if not guide.lowest_addr or saddr < guide.lowest_addr:
-                    guide.lowest_addr = saddr
-                if not guide.highest_addr or eaddr > guide.highest_addr:
-                    guide.highest_addr = eaddr
-
-                if not last_addr:
-                    last_addr = eaddr
-                    continue
-                if last_addr != saddr:
-                    gaps.append([last_addr, saddr])
-                last_addr = eaddr
-
-            if not guide.gaps:
-                guide.gaps = gaps
-            else:
-                guide.gaps = overlapping_regions(guide.gaps, gaps)
-
-    return sorted(list(guides.values()), key=lambda x: x.total_space(),
-                    reverse=True)
-
-def pr_guide(records):
-    for guide in get_guide_info(records):
-        print(guide)
-
-def region_sort_key(region):
-    return region[1] - region[0]
-
 def set_missed_args(args, records):
     if args.tid and args.time_range and args.address_range:
         return
-    guides = get_guide_info(records)
+    guides = damo_record_info.get_guide_info(records)
     guide = guides[0]
     if not args.tid:
         args.tid = guide.tid
@@ -316,7 +239,10 @@ def plot_heatmap(data_file, output_file, args):
     set ylabel 'Address (bytes)';
     plot '%s' using 1:2:3 with image;""" % (terminal, output_file, x_range[0],
             x_range[1], y_range[0], y_range[1], data_file)
-    subprocess.call(['gnuplot', '-e', gnuplot_cmd])
+    try:
+        subprocess.call(['gnuplot', '-e', gnuplot_cmd])
+    except Exception as e:
+        print('executing gnuplot failed (%s)' % e)
     os.remove(data_file)
 
 def set_argparser(parser):
@@ -349,14 +275,12 @@ def set_argparser(parser):
             help='another name of stdout_heatmap_color')
     parser.add_argument('--plot_ascii', action='store_true',
             help='shortcut of \'--heatmap stdout\'')
-    parser.description='Show when which address ranges were how frequently accessed'
+    parser.add_argument('--stdout_heatmap_skip_color_example',
+            action='store_true',
+            help='skip printing example colors at the output')
+    parser.description = 'Show when which address ranges were how frequently accessed'
 
 def main(args=None):
-    if not args:
-        parser = argparse.ArgumentParser()
-        set_argparser(parser)
-        args = parser.parse_args()
-
     # --plot_ascii and --ascii_color is used in the demo screenshop[1].
     # Support those.
     #
@@ -368,7 +292,7 @@ def main(args=None):
     if args.ascii_color == None and args.stdout_heatmap_color == None:
         args.stdout_heatmap_color = 'gray'
 
-    records, err = _damon_result.parse_records_file(args.input)
+    records, err = _damon_records.get_records(record_file=args.input)
     if err != None:
         print('monitoring result file (%s) parsing failed (%s)' %
                 (args.input, err))
@@ -379,22 +303,20 @@ def main(args=None):
         args.resol = [40, 80]
 
     if args.guide:
-        pr_guide(records)
-    else:
-        set_missed_args(args, records)
-        orig_stdout = sys.stdout
-        if args.heatmap and args.heatmap != 'stdout':
-            tmp_path = tempfile.mkstemp()[1]
-            tmp_file = open(tmp_path, 'w')
-            sys.stdout = tmp_file
+        damo_record_info.pr_guide(records)
+        return
 
-        pr_heats(args, records)
+    set_missed_args(args, records)
+    orig_stdout = sys.stdout
+    if args.heatmap and args.heatmap != 'stdout':
+        tmp_path = tempfile.mkstemp()[1]
+        tmp_file = open(tmp_path, 'w')
+        sys.stdout = tmp_file
 
-        if args.heatmap and args.heatmap != 'stdout':
-            sys.stdout = orig_stdout
-            tmp_file.flush()
-            tmp_file.close()
-            plot_heatmap(tmp_path, args.heatmap, args)
+    pr_heats(args, records)
 
-if __name__ == '__main__':
-    main()
+    if args.heatmap and args.heatmap != 'stdout':
+        sys.stdout = orig_stdout
+        tmp_file.flush()
+        tmp_file.close()
+        plot_heatmap(tmp_path, args.heatmap, args)
