@@ -19,22 +19,7 @@ def sighandler(signum, frame):
     print('\nsignal %s received' % signum)
     cleanup()
 
-def set_argparser(parser):
-    parser.add_argument('target', type=str, metavar='<target>',
-            help='monitoring target (command, pid or \'paddr\')')
-    parser.add_argument('--report_type', type=str, choices=['heats', 'wss'],
-            default='heats', help='report type')
-    parser.add_argument('--delay', type=float, metavar='<seconds>', default=3,
-            help='deplay between updates in seconds.')
-    parser.add_argument('--count', type=int, metavar='<count>', default=0,
-            help='number of updates.')
-
-def main(args=None):
-    if not args:
-        parser = argparse.ArgumentParser()
-        set_argparser(parser)
-        args = parser.parse_args()
-
+def main(args):
     _damon.ensure_root_permission()
 
     global target_type
@@ -93,5 +78,12 @@ def main(args=None):
 
     cleanup()
 
-if __name__ == '__main__':
-    main()
+def set_argparser(parser):
+    parser.add_argument('target', type=str, metavar='<target>',
+            help='monitoring target (command, pid or \'paddr\')')
+    parser.add_argument('--report_type', type=str, choices=['heats', 'wss'],
+            default='heats', help='report type')
+    parser.add_argument('--delay', type=float, metavar='<seconds>', default=3,
+            help='deplay between updates in seconds.')
+    parser.add_argument('--count', type=int, metavar='<count>', default=0,
+            help='number of updates.')
