@@ -11,16 +11,21 @@ import time
 
 import _damon
 import _damon_args
-import _damon_result
+import _damon_records
 
 class DataForCleanup:
     kdamonds_idxs = None
     orig_kdamonds = None
-    perf_pipe = None
+    record_handle = None
 
 data_for_cleanup = DataForCleanup()
 
+cleaning = False
 def cleanup_exit(exit_code):
+    global cleaning
+    if cleaning == True:
+        return
+    cleaning = True
     if data_for_cleanup.kdamonds_idxs != None:
         # ignore returning error, as kdamonds may already finished
         _damon.turn_damon_off(data_for_cleanup.kdamonds_idxs)
@@ -28,8 +33,8 @@ def cleanup_exit(exit_code):
         if err:
             print('failed restoring previous kdamonds setup (%s)' % err)
 
-    if data_for_cleanup.perf_pipe:
-        _damon_result.stop_monitoring_record(data_for_cleanup.perf_pipe)
+    if data_for_cleanup.record_handle:
+        _damon_records.finish_recording(data_for_cleanup.record_handle)
 
     exit(exit_code)
 
@@ -41,7 +46,7 @@ def handle_args(args):
     if _damon.any_kdamond_running() and not args.deducible_target:
         args.deducible_target = 'ongoing'
 
-    args.output_permission, err = _damon_result.parse_file_permission_str(
+    args.output_permission, err = _damon_records.parse_file_permission_str(
             args.output_permission)
     if err != None:
         print('wrong --output permission (%s) (%s)' %
@@ -52,7 +57,7 @@ def handle_args(args):
     if os.path.isfile(args.out):
         os.rename(args.out, args.out + '.old')
 
-    err = _damon_result.set_perf_path(args.perf_path)
+    err = _damon_records.set_perf_path(args.perf_path)
     if err != None:
         print(err)
         exit(-3)
@@ -115,29 +120,13 @@ def poll_target_pids(kdamonds, add_childs):
             cleanup_exit(1)
     return True
 
-def set_argparser(parser):
-    parser = _damon_args.set_argparser(parser, add_record_options=True)
-    parser.add_argument('--output_type',
-            choices=_damon_result.file_types,
-            default=_damon_result.file_type_json_compressed,
-            help='output file\'s type')
-    parser.add_argument('--output_permission', type=str, default='600',
-            help='permission of the output file')
-    parser.add_argument('--perf_path', type=str, default='perf',
-            help='path of perf tool ')
-    parser.add_argument('--include_child_tasks', action='store_true',
-            help='record accesses of child processes')
-    parser.description='Record monitoring results'
-    return parser
-
-def main(args=None):
+def main(args):
     global data_for_cleanup
 
-    if not args:
-        parser = set_argparser(None)
-        args = parser.parse_args()
-
-    _damon.ensure_root_and_initialized(args)
+    is_ongoing = _damon_args.is_ongoing_target(args)
+    _damon.ensure_root_and_initialized(args,
+            load_feature_supports=is_ongoing,
+            save_feature_supports=not is_ongoing)
 
     handle_args(args)
 
@@ -147,7 +136,6 @@ def main(args=None):
     signal.signal(signal.SIGTERM, sighandler)
 
     # Now the real works
-    is_ongoing = _damon_args.is_ongoing_target(args)
     if not is_ongoing:
         err, kdamonds = _damon_args.turn_damon_on(args)
         if err:
@@ -166,9 +154,15 @@ def main(args=None):
         monitoring_intervals = data_for_cleanup.orig_kdamonds[
                 0].contexts[0].intervals
 
-    data_for_cleanup.perf_pipe = _damon_result.start_monitoring_record(
-            args.out, args.output_type, args.output_permission,
-            monitoring_intervals)
+    if args.schemes_target_regions == False:
+        tracepoint = _damon_records.perf_event_damon_aggregated
+    else:
+        tracepoint = _damon_records.perf_event_damos_before_apply
+
+    data_for_cleanup.record_handle = _damon_records.start_recording(
+            tracepoint, args.out, args.output_type, args.output_permission,
+            monitoring_intervals,
+            profile=args.profile is True, profile_target_pid=None)
     print('Press Ctrl+C to stop')
 
     if _damon_args.self_started_target(args):
@@ -179,5 +173,21 @@ def main(args=None):
 
     cleanup_exit(0)
 
-if __name__ == '__main__':
-    main()
+def set_argparser(parser):
+    parser = _damon_args.set_argparser(parser, add_record_options=True)
+    parser.add_argument('--output_type',
+                        choices=_damon_records.file_types,
+                        default=_damon_records.file_type_json_compressed,
+                        help='output file\'s type')
+    parser.add_argument('--output_permission', type=str, default='600',
+                        help='permission of the output file')
+    parser.add_argument('--perf_path', type=str, default='perf',
+                        help='path of perf tool ')
+    parser.add_argument('--include_child_tasks', action='store_true',
+                        help='record accesses of child processes')
+    parser.add_argument('--schemes_target_regions', action='store_true',
+                        help='record schemes tried to be applied regions')
+    parser.add_argument('--profile', action='store_true',
+                        help='record profiling information together')
+    parser.description = 'Record monitoring results'
+    return parser
