@@ -7,7 +7,7 @@ Print basic information of the access monitoring results record file.
 import argparse
 
 import _damo_fmt_str
-import _damon_result
+import _damon_records
 
 class GuideInfo:
     tid = None
@@ -115,25 +115,14 @@ def pr_guide(records):
     for guide in get_guide_info(records):
         print(guide)
 
-def region_sort_key(region):
-    return region[1] - region[0]
-
-def set_argparser(parser):
-    parser.add_argument('--input', '-i', type=str, metavar='<file>',
-            default='damon.data', help='input file name')
-
-def main(args=None):
-    if not args:
-        parser = argparse.ArgumentParser()
-        set_argparser(parser)
-        args = parser.parse_args()
-
-    records, err = _damon_result.parse_records_file(args.input)
+def main(args):
+    records, err = _damon_records.get_records(record_file=args.input)
     if err != None:
         print('monitoring result file (%s) parsing failed (%s)' %
                 (args.input, err))
         exit(1)
     pr_guide(records)
 
-if __name__ == '__main__':
-    main()
+def set_argparser(parser):
+    parser.add_argument('--input', '-i', type=str, metavar='<file>',
+            default='damon.data', help='input file name')
