@@ -7,16 +7,7 @@ Stop DAMON.
 import _damon
 import _damon_args
 
-def set_argparser(parser):
-    _damon_args.set_common_argparser(parser)
-    parser.description='Stop DAMON'
-    return parser
-
-def main(args=None):
-    if not args:
-        parser = set_argparser(parser)
-        args = parser.parse_args()
-
+def main(args):
     _damon.ensure_root_and_initialized(args)
 
     running_kdamond_idxs = _damon.running_kdamond_idxs()
@@ -29,5 +20,7 @@ def main(args=None):
         print('DAMON turn off failed (%s)' % err)
         exit(1)
 
-if __name__ == '__main__':
-    main()
+def set_argparser(parser):
+    _damon_args.set_common_argparser(parser)
+    parser.description = 'Stop DAMON'
+    return parser
