@@ -5,7 +5,7 @@
 import argparse
 import os
 
-import _damon_result
+import _damon_records
 
 def assert_value_in_range(value, min_max, name, error_allowed):
     '''Returns 0 if the value is in the range, 1 if the value is out of range
@@ -32,30 +32,7 @@ def check_boundary(region, regions_boundary):
                 (region.start, region.end))
         exit(1)
 
-
-def set_argparser(parser):
-    parser.add_argument('--input', '-i', type=str, metavar='<file>',
-            default='damon.data', help='input file name')
-    parser.add_argument('--aggr', metavar='<microseconds>', type=int, nargs=2,
-            default=[80000, 120000],
-            help='min/max valid sample intervals (us)')
-    parser.add_argument('--nr_regions', metavar='<number of regions>',
-            type=int, nargs=2, default=[3, 1200],
-            help='min/max number of regions')
-    parser.add_argument('--nr_accesses', metavar='<number of accesses>',
-            type=int, nargs=2, default=[0, 24],
-            help='min/max number of measured accesses per aggregate interval')
-    parser.add_argument('--allow_error', metavar='<percent>', default=2,
-            help='allowed percent of error samples')
-    parser.add_argument('--regions_boundary', metavar='<start>-<end>',
-            nargs='+', help='regions boundary')
-
-def main(args=None):
-    if not args:
-        parser = argparse.ArgumentParser()
-        set_argparser(parser)
-        args = parser.parse_args()
-
+def main(args):
     if not os.path.isfile(args.input):
         print('the file (%s) not found' % args.input)
         exit(1)
@@ -68,7 +45,7 @@ def main(args=None):
                 print('wrong boundary input %s' % boundary)
             regions_boundary.append(parsed_boundary)
 
-    records, err = _damon_result.parse_records_file(args.input)
+    records, err = _damon_records.get_records(record_file=args.input)
     if err != None:
         print('parsing failed (%s)' % err)
         exit(1)
@@ -104,5 +81,19 @@ def main(args=None):
                 assert_value_in_range(region.nr_accesses.samples,
                         args.nr_accesses, 'nr_accesses', False)
 
-if __name__ == '__main__':
-    main()
+def set_argparser(parser):
+    parser.add_argument('--input', '-i', type=str, metavar='<file>',
+            default='damon.data', help='input file name')
+    parser.add_argument('--aggr', metavar='<microseconds>', type=int, nargs=2,
+            default=[80000, 120000],
+            help='min/max valid sample intervals (us)')
+    parser.add_argument('--nr_regions', metavar='<number of regions>',
+            type=int, nargs=2, default=[3, 1200],
+            help='min/max number of regions')
+    parser.add_argument('--nr_accesses', metavar='<number of accesses>',
+            type=int, nargs=2, default=[0, 24],
+            help='min/max number of measured accesses per aggregate interval')
+    parser.add_argument('--allow_error', metavar='<percent>', default=2,
+            help='allowed percent of error samples')
+    parser.add_argument('--regions_boundary', metavar='<start>-<end>',
+            nargs='+', help='regions boundary')
