@@ -8,7 +8,7 @@ import tempfile
 
 import _damo_dist
 import _damo_fmt_str
-import _damon_result
+import _damon_records
 
 def get_wss_dists(records, acc_thres, sz_thres, do_sort):
     wss_dists = {}
@@ -100,14 +100,9 @@ def set_argparser(parser):
             help='use machine-friendly raw numbers')
     parser.add_argument('--all_wss', action='store_true',
             help='Do not print percentile but all calculated wss')
-    parser.description='Show distribution of working set size'
-
-def main(args=None):
-    if not args:
-        parser = argparse.ArgumentParser()
-        set_argparser(parser)
-        args = parser.parse_args()
+    parser.description = 'Show distribution of working set size'
 
+def main(args):
     file_path = args.input
     percentiles = range(args.range[0], args.range[1], args.range[2])
     wss_sort = True
@@ -115,13 +110,13 @@ def main(args=None):
         wss_sort = False
     raw_number = args.raw_number
 
-    records, err = _damon_result.parse_records_file(file_path)
+    records, err = _damon_records.get_records(record_file=file_path)
     if err != None:
         print('monitoring result file (%s) parsing failed (%s)' %
                 (file_path, err))
         exit(1)
 
-    _damon_result.adjust_records(records, args.work_time, args.exclude_samples)
+    _damon_records.adjust_records(records, args.work_time, args.exclude_samples)
     wss_dists = get_wss_dists(records, args.acc_thres, args.sz_thres, wss_sort)
 
     if args.plot:
@@ -146,6 +141,3 @@ def main(args=None):
                 'working set size (bytes)')
         if err:
             print('plot failed (%s)' % err)
-
-if __name__ == '__main__':
-    main()
