@@ -24,7 +24,14 @@ fi
 for damon_interface in $damon_interfaces
 do
 	testname2="$testname $damon_interface"
-	sudo "$damo" start --ops paddr --damon_interface "$damon_interface" \
+	if sudo "$damo" features supported | grep schemes_tried_regions --quiet
+	then
+		do_show_test="true"
+	else
+		do_show_test="false"
+	fi
+	sudo "$damo" start --ops paddr \
+		--damon_interface_DEPRECATED "$damon_interface" \
 		-c monitoring_damos.json 2> /dev/null
 	if ! pgrep kdamond.0 > /dev/null
 	then
@@ -34,7 +41,7 @@ do
 	echo "PASS $testname2 start"
 
 	sudo timeout 3 "$damo" record ongoing \
-		--damon_interface "$damon_interface" &> /dev/null
+		--damon_interface_DEPRECATED "$damon_interface" &> /dev/null
 	if ! "$damo" validate 2> /dev/null
 	then
 		echo "FAIL $testname2 (invalid record file)"
@@ -60,8 +67,25 @@ do
 	done
 	echo "PASS $testname2 status $i"
 
+	if [ "$do_show_test" = "true" ]
+	then
+		for i in {1..10}
+		do
+			if ! sudo "$damo" show &> /dev/null
+			then
+				echo "FAIL $testname2 show $i failed"
+				if ! sudo "$damo" stop
+				then
+					echo "failed stopping DAMON"
+				fi
+				exit 1
+			fi
+		done
+		echo "PASS $testname2 show $i"
+	fi
+
 	if ! sudo "$damo" tune --aggr 200000 --ops paddr \
-		--damon_interface "$damon_interface" &> /dev/null
+		--damon_interface_DEPRECATED "$damon_interface" &> /dev/null
 	then
 		echo "FAIL $testname2 tune"
 		if ! sudo "$damo" stop
@@ -72,7 +96,7 @@ do
 		exit 1
 	fi
 	sudo timeout 3 "$damo" record ongoing \
-		--damon_interface "$damon_interface" &> /dev/null
+		--damon_interface_DEPRECATED "$damon_interface" &> /dev/null
 	if ! "$damo" validate --aggr 180000 220000 2> /dev/null
 	then
 		echo "FAIL $testname2 (invalid record file after tune)"
@@ -84,7 +108,39 @@ do
 	fi
 	echo "PASS $testname2 tune-record-ongoing-validate"
 
-	sudo "$damo" stop --damon_interface "$damon_interface" 2> /dev/null
+	for i in {1..10}
+	do
+		if ! sudo "$damo" status &> /dev/null
+		then
+			echo "FAIL $testname2 tune-status $i failed"
+			if ! sudo "$damo" stop
+			then
+				echo "failed stopping DAMON"
+			fi
+			exit 1
+		fi
+	done
+	echo "PASS $testname2 tune-status $i"
+
+	if [ "$do_show_test" = "true" ]
+	then
+		for i in {1..10}
+		do
+			if ! sudo "$damo" show &> /dev/null
+			then
+				echo "FAIL $testname2 tune-show $i failed"
+				if ! sudo "$damo" stop
+				then
+					echo "failed stopping DAMON"
+				fi
+				exit 1
+			fi
+		done
+		echo "PASS $testname2 tune-show $i"
+	fi
+
+	sudo "$damo" stop --damon_interface_DEPRECATED "$damon_interface" \
+		2> /dev/null
 	if pgrep kdamond.0 > /dev/null
 	then
 		echo "FAIL $testname2 (kdamond.0 pid found after stop)"
