@@ -26,6 +26,7 @@ class TestDamon(unittest.TestCase):
                 _damon.DamosAccessPattern([0, 10], [5, 8], _damon.unit_percent,
                     [54, 88], _damon.unit_usec),
                 'pageout',
+                None,
                 _damon.DamosQuotas(100, 1024, 1000, [80, 76, 24]),
                 _damon.DamosWatermarks('free_mem_rate', 5000000, 800, 500,
                     200),
@@ -35,19 +36,21 @@ class TestDamon(unittest.TestCase):
         damos_kvpairs = damos.to_kvpairs()
         self.assertEqual(type(damos_kvpairs), collections.OrderedDict)
         self.assertEqual(list(damos_kvpairs.keys()),
-                ['action', 'access_pattern', 'quotas', 'watermarks',
-                    'filters'])
+                ['action', 'access_pattern', 'apply_interval_us', 'quotas',
+                    'watermarks', 'filters', 'stats'])
+        self.assertEqual(list(damos_kvpairs['stats'].keys()),
+                ['nr_tried', 'sz_tried', 'nr_applied', 'sz_applied',
+                    'qt_exceeds'])
         self.assertEqual(damos, _damon.Damos.from_kvpairs(damos_kvpairs))
 
-        ctx = _damon.DamonCtx(
+        ctx = _damon.DamonCtx('paddr', [target],
                 _damon.DamonIntervals(5000, 100000, 1000000),
                 _damon.DamonNrRegionsRange(10, 1000),
-                'paddr', [target], [damos])
+                [damos])
         ctx_kvpairs = ctx.to_kvpairs()
         self.assertEqual(type(ctx_kvpairs), collections.OrderedDict)
         self.assertEqual(list(ctx_kvpairs.keys()),
-                ['intervals', 'nr_regions', 'ops', 'targets',
-                    'schemes'])
+                ['ops', 'targets', 'intervals', 'nr_regions', 'schemes'])
         self.assertEqual(ctx, _damon.DamonCtx.from_kvpairs(ctx_kvpairs))
 
         kdamond = _damon.Kdamond('off', 123, [ctx])
@@ -71,6 +74,7 @@ class TestDamon(unittest.TestCase):
                     18446744073709551615], [0.0, 0.0], _damon.unit_percent,
                     [1000000, 4294900000], _damon.unit_usec),
                 action='stat',
+                apply_interval_us=None,
                 quotas=_damon.DamosQuotas(time_ms=0, sz_bytes=584792941,
                     reset_interval_ms=1000, weights=[0,0,0]),
                 watermarks=_damon.DamosWatermarks(
