@@ -14,14 +14,14 @@ import _damon_sysfs
 
 class TestDamonArgs(unittest.TestCase):
     def test_damon_ctx_for(self):
-        _damon_sysfs.feature_supports = {'init_regions': True, 'schemes': True,
+        _damon._damon_fs = _damon_sysfs
+        _damon.set_feature_supports({'init_regions': True, 'schemes': True,
                 'schemes_stat_qt_exceed': True, 'init_regions_target_idx':
                 True, 'schemes_prioritization': True, 'schemes_tried_regions':
                 False, 'record': False, 'schemes_quotas': True, 'fvaddr':
                 False, 'paddr': True, 'schemes_wmarks': True,
                 'schemes_speed_limit': True, 'schemes_stat_succ': True,
-                'vaddr': True}
-        _damon._damon_fs = _damon_sysfs
+                'vaddr': True})
 
         parser = argparse.ArgumentParser()
         _damon_args.set_argparser(parser, add_record_options=False)
@@ -33,19 +33,15 @@ class TestDamonArgs(unittest.TestCase):
         self.assertEqual(err, None)
         ctx, err = _damon_args.damon_ctx_for(args)
         self.assertEqual(err, None)
-        self.assertEqual(ctx,
-            _damon.DamonCtx(
-                _damon.DamonIntervals(5000, 100000, 1000000),
-                _damon.DamonNrRegionsRange(10, 1000), 'paddr',
-                [_damon.DamonTarget(None,
-                    [_damon.DamonRegion(123, 456)])], []))
+        self.assertEqual(ctx, _damon.DamonCtx('paddr',
+            [_damon.DamonTarget(None, [_damon.DamonRegion(123, 456)])],
+            _damon.DamonIntervals(5000, 100000, 1000000),
+            _damon.DamonNrRegionsRange(10, 1000), []))
 
-        self.assertEqual(ctx,
-            _damon.DamonCtx(
-                _damon.DamonIntervals(5000, 100000, 1000000),
-                _damon.DamonNrRegionsRange(10, 1000), 'paddr',
-                [_damon.DamonTarget(None,
-                    [_damon.DamonRegion(123, 456)])], []))
+        self.assertEqual(ctx, _damon.DamonCtx('paddr',
+            [_damon.DamonTarget(None, [_damon.DamonRegion(123, 456)])],
+            _damon.DamonIntervals(5000, 100000, 1000000),
+            _damon.DamonNrRegionsRange(10, 1000), []))
 
         args = parser.parse_args(
                 ('--sample 5ms --aggr 100ms --updr 1s ' +
@@ -54,13 +50,10 @@ class TestDamonArgs(unittest.TestCase):
         self.assertEqual(err, None)
         ctx, err = _damon_args.damon_ctx_for(args)
         self.assertEqual(err, None)
-        self.assertEqual(ctx,
-            _damon.DamonCtx(
-                _damon.DamonIntervals(5000, 100000, 1000000),
-                _damon.DamonNrRegionsRange(10, 1000), 'paddr',
-                [_damon.DamonTarget(None,
-                    [_damon.DamonRegion(1024, 4096)])],
-                []))
+        self.assertEqual(ctx, _damon.DamonCtx('paddr',
+            [_damon.DamonTarget(None, [_damon.DamonRegion(1024, 4096)])],
+            _damon.DamonIntervals(5000, 100000, 1000000),
+            _damon.DamonNrRegionsRange(10, 1000), []))
 
         parser = argparse.ArgumentParser()
         _damon_args.set_argparser(parser, add_record_options=False)
@@ -71,17 +64,15 @@ class TestDamonArgs(unittest.TestCase):
                     '--ops paddr').split())
         ctx, err = _damon_args.damon_ctx_for(args)
         self.assertEqual(err, None)
-        self.assertEqual(ctx,
-            _damon.DamonCtx(
-                _damon.DamonIntervals(5000, 100000, 1000000),
-                _damon.DamonNrRegionsRange(10, 1000), 'paddr',
-                [_damon.DamonTarget(None,
-                    [_damon.DamonRegion(1024, 4096)])],
-                []))
+        self.assertEqual(ctx, _damon.DamonCtx('paddr',
+            [_damon.DamonTarget(None, [_damon.DamonRegion(1024, 4096)])],
+            _damon.DamonIntervals(5000, 100000, 1000000),
+            _damon.DamonNrRegionsRange(10, 1000), []))
 
     def test_damon_intervals_for(self):
         parser = argparse.ArgumentParser()
         _damon_args.set_monitoring_attrs_argparser(parser)
+        _damon_args.set_monitoring_attrs_pinpoint_argparser(parser)
 
         args = parser.parse_args(
                 '--monitoring_intervals 4ms 120ms 1.5s'.split())
@@ -96,6 +87,7 @@ class TestDamonArgs(unittest.TestCase):
     def test_damon_nr_regions_range_for(self):
         parser = argparse.ArgumentParser()
         _damon_args.set_monitoring_attrs_argparser(parser)
+        _damon_args.set_monitoring_attrs_pinpoint_argparser(parser)
 
         args = parser.parse_args(
                 '--monitoring_nr_regions_range 25 5000'.split())
